module Constants
  DB_URL = "https://mtgjson.com/api/v5/AllPrintings.sqlite"
  DECK_FILE_PATH = "deck.txt"
  DB_FILE_NAME = "AllPrintings.sqlite"
end.freeze


module Mtg
  # Safe SQL query builder to prevent SQL injection
  class QueryBuilder
    def initialize
      @select_clause = []
      @from_clause = nil
      @join_clauses = []
      @where_clauses = []
      @group_by_clause = []
      @order_by_clause = []
      @limit_clause = nil
      @params = []
    end

    def select(*fields)
      @select_clause.concat(fields)
      self
    end

    def from(table)
      @from_clause = table
      self
    end

    def join(table, on_condition)
      @join_clauses << "JOIN #{table} ON #{on_condition}"
      self
    end

    def left_join(table, on_condition)
      @join_clauses << "LEFT JOIN #{table} ON #{on_condition}"
      self
    end

    def where(condition, *params)
      @where_clauses << condition
      @params.concat(params)
      self
    end

    def where_like(column, value)
      @where_clauses << "#{column} LIKE ?"
      @params << value
      self
    end

    def where_in(column, values)
      placeholders = values.map { "?" }.join(", ")
      @where_clauses << "#{column} IN (#{placeholders})"
      @params.concat(values)
      self
    end

    def where_colors(column, colors)
      color_conditions = colors.map do |color|
        @params << "%#{color}%"
        "#{column} LIKE ?"
      end
      @where_clauses << "(#{color_conditions.join(" OR ")})"
      self
    end

    def where_text_keywords(column, keywords)
      keyword_conditions = keywords.map do |keyword|
        @params << "%#{keyword}%"
        "#{column} LIKE ?"
      end
      @where_clauses << "(#{keyword_conditions.join(" OR ")})"
      self
    end

    def where_max_value(column, max_value)
      return self unless max_value
      @where_clauses << "#{column} <= ?"
      @params << max_value
      self
    end

    def group_by(*fields)
      @group_by_clause.concat(fields)
      self
    end

    def order_by(*fields)
      @order_by_clause.concat(fields)
      self
    end

    def limit(count)
      @limit_clause = count
      self
    end

    def to_sql
      sql_parts = []

      # SELECT clause
      select_fields = @select_clause.empty? ? ["*"] : @select_clause
      sql_parts << "SELECT #{select_fields.join(", ")}"

      # FROM clause
      sql_parts << "FROM #{@from_clause}" if @from_clause

      # JOIN clauses
      sql_parts.concat(@join_clauses)

      # WHERE clause
      unless @where_clauses.empty?
        sql_parts << "WHERE #{@where_clauses.join(" AND ")}"
      end

      # GROUP BY clause
      unless @group_by_clause.empty?
        sql_parts << "GROUP BY #{@group_by_clause.join(", ")}"
      end

      # ORDER BY clause
      unless @order_by_clause.empty?
        sql_parts << "ORDER BY #{@order_by_clause.join(", ")}"
      end

      # LIMIT clause
      sql_parts << "LIMIT #{@limit_clause}" if @limit_clause

      sql_parts.join("\n")
    end

    attr_reader :params
  end
end


module Mtg
  module Database
    # Abstract base class for database adapters
    # Defines the interface that all database implementations must follow
    class Adapter
      # Execute a SQL query with optional parameters
      # @param query [String] SQL query with ? placeholders for parameters
      # @param params [Array] Parameters to bind to the query
      # @return [Array<Hash>] Array of hashes representing rows
      def execute(query, params = [])
        raise NotImplementedError, "Subclasses must implement #execute"
      end

      # Prepare a statement for repeated execution
      # @param query [String] SQL query with ? placeholders
      # @return [PreparedStatement] A prepared statement object
      def prepare_statement(query)
        raise NotImplementedError, "Subclasses must implement #prepare_statement"
      end

      # Close the database connection
      def close
        raise NotImplementedError, "Subclasses must implement #close"
      end

      # Check if the database connection is open
      # @return [Boolean]
      def connected?
        raise NotImplementedError, "Subclasses must implement #connected?"
      end
    end

    # Abstract base class for prepared statements
    class PreparedStatement
      # Execute the prepared statement with given parameters
      # @param params [Array] Parameters to bind to the statement
      # @return [Array<Hash>] Array of hashes representing rows
      def execute(params = [])
        raise NotImplementedError, "Subclasses must implement #execute"
      end

      # Close the prepared statement
      def close
        raise NotImplementedError, "Subclasses must implement #close"
      end
    end
  end
end



module Mtg
  module Database
    # SQL.js adapter implementation for web browser usage
    # This adapter expects the SQL.js database object to be passed in
    class SqljsAdapter < Adapter
      def initialize(db)
        @db = db
      end

      def execute(query, params = [])
        # Replace ? placeholders with actual values for SQL.js
        # SQL.js doesn't support parameterized queries in exec
        if params.any?
          param_index = 0
          query = query.gsub("?") do
            value = params[param_index]
            param_index += 1
            if value.is_a?(String)
              "'#{value.gsub("'", "''")}'"
            elsif value.nil?
              "NULL"
            else
              value.to_s
            end
          end
        end

        # SQL.js exec returns an array of result objects
        # Each result has columns and values arrays
        results = @db.exec(query)

        # Convert JS array to Ruby array and check if empty
        results_array = results.to_a
        return [] if results_array.empty? || results_array.first.nil?

        # Convert SQL.js format to array of hashes
        result = results_array.first
        columns = result[:columns].to_a
        values = result[:values].to_a

        rows = []
        values.each do |row_values|
          row_hash = {}
          # Convert JS array to Ruby array
          row_values_array = row_values.to_a
          columns.each_with_index do |col, idx|
            # Convert column name to string and get value
            col_name = col.to_s
            value = row_values_array[idx]
            # Convert JS values to Ruby values where needed
            row_hash[col_name] = if value.nil?
              nil
            elsif value.is_a?(JS::Object)
              # Try to convert to appropriate Ruby type
              value.to_s
            else
              value
            end
          end
          rows << row_hash
        end

        rows
      end

      def prepare_statement(query)
        SqljsPreparedStatement.new(@db, query)
      end

      def close
        @db&.close
      end

      def connected?
        # SQL.js doesn't have a specific "closed" state
        # If the db object exists, consider it connected
        !@db.nil?
      end
    end

    # SQL.js prepared statement implementation
    class SqljsPreparedStatement < PreparedStatement
      def initialize(db, query)
        @db = db
        @query = query
        # SQL.js uses prepare() method
        @stmt = @db.prepare(query)
      end

      def execute(params = [])
        # Bind parameters
        params.each_with_index do |param, idx|
          # SQL.js uses 1-based indexing for parameters
          @stmt.bind(idx + 1, param)
        end

        rows = []

        # Step through results
        while @stmt.step
          # getAsObject returns the current row as a JavaScript object
          row_object = @stmt.getAsObject
          row_hash = {}

          # Convert JavaScript object to Ruby hash
          JS.global[:Object].keys(row_object).to_a.each do |key|
            key_str = key.to_s
            value = row_object[key]
            # Convert JS values to Ruby values where needed
            row_hash[key_str] = value.is_a?(JS::Object) ? value.to_s : value
          end

          rows << row_hash
        end

        # Reset for next execution
        @stmt.reset

        rows
      end

      def close
        @stmt&.free
      end
    end
  end
end



module Mtg
  class Query
    def initialize(db_adapter)
      @db = db_adapter
    end

    # 日本語のカード名で検索して詳細情報を取得
    def find_card_by_jp_name(jp_name)
      query = QueryBuilder.new
        .select(
          "cf.name as jp_name",
          "c.name as en_name",
          "c.manaCost",
          "c.manaValue",
          "c.colors",
          "c.type",
          "cf.text as jp_text",
          "c.text as en_text",
          "c.rarity",
          "c.setCode"
        )
        .from("cardForeignData cf")
        .join("cards c", "cf.uuid = c.uuid")
        .where("cf.language = ?", "Japanese")
        .where("cf.name = ?", jp_name)
        .group_by("c.name")
        .limit(1)

      results = @db.execute(query.to_sql, query.params)
      results.first
    end

    # 色とテキストキーワードから検索
    def search_cards_by_colors_and_text(colors, text_keywords, max_cmc = nil)
      query = QueryBuilder.new
        .select(
          "c.name",
          "c.manaCost",
          "c.manaValue",
          "c.colors",
          "c.type",
          "c.text",
          "c.rarity",
          "cf.name as jp_name",
          "cf.text as jp_text"
        )
        .from("cards c")
        .left_join("cardForeignData cf", "c.uuid = cf.uuid and cf.language = 'Japanese'")
        .where_colors("c.colors", colors)
        .where_text_keywords("c.text", text_keywords)
        .where_max_value("c.manaValue", max_cmc)
        .group_by("c.name")
        .order_by("c.manaValue", "c.name")
        .limit(30)

      @db.execute(query.to_sql, query.params)
    end

    # タイプで検索
    def search_cards_by_type(type_keyword, colors = [], max_cmc = nil)
      query = QueryBuilder.new
        .select(
          "c.name",
          "c.manaCost",
          "c.manaValue",
          "c.colors",
          "c.type",
          "c.text",
          "c.rarity",
          "cf.name as jp_name",
          "cf.text as jp_text"
        )
        .from("cards c")
        .left_join("cardForeignData cf", "c.uuid = cf.uuid and cf.language = 'Japanese'")
        .where_like("c.type", "%#{type_keyword}%")

      # Add color filter if specified
      query.where_colors("c.colors", colors) if colors.any?

      # Add CMC filter if specified
      query.where_max_value("c.manaValue", max_cmc)

      query
        .group_by("c.name")
        .order_by("c.manaValue", "c.colors", "c.name")
        .limit(30)

      @db.execute(query.to_sql, query.params)
    end

    # 日本語カード名で部分一致検索
    def search_cards_by_jp_name(name)
      query = QueryBuilder.new
        .select(
          "cf.name as jp_name",
          "c.name as en_name",
          "c.manaCost",
          "c.manaValue",
          "c.colors",
          "c.type",
          "cf.text as jp_text",
          "c.text as en_text",
          "c.rarity",
          "c.setCode"
        )
        .from("cardForeignData cf")
        .join("cards c", "cf.uuid = c.uuid")
        .where("cf.language = ?", "Japanese")
        .where_like("cf.name", "%#{name}%")
        .group_by("c.name")
        .order_by("c.name")
        .limit(30)

      @db.execute(query.to_sql, query.params)
    end
  end
end



module Mtg
  class Service
    attr_reader :deck_cards

    def initialize(query)
      @query = query
      @deck_cards = [] # デッキのカードリスト [{count: 4, card: {...}}]
    end

    def load_deck_from_text!(text)
      lines = text.split("\n").map(&:strip).reject(&:empty?)

      # Clear previous deck
      @deck_cards = []

      lines.each do |line|
        if line =~ /^(\d+)x?\s+(.+)$/
          count = $1.to_i
          name = $2
        else
          count = 1
          name = line
        end

        card = @query.find_card_by_jp_name(name)
        if card
          @deck_cards << {count: count, card: card}
        elsif defined?(JS) && JS.respond_to?(:global)
          # Web environment can't use warn, use console instead
          JS.global[:console].warn("警告: カードが見つかりません: #{name}")
        else
          warn "警告: カードが見つかりません: #{name}"
        end
      end
    end

    def load_deck_from_file!(file_path)
      content = File.read(file_path)
      load_deck_from_text!(content)
    end

    # デッキを統計的に分析
    def analyze_deck
      if @deck_cards.empty?
        return {
          card_count: 0,
          mana_curve: {},
          color_distribution: {},
          type_distribution: {}
        }
      end

      total_cards = 0
      mana_curve = Hash.new(0)
      color_distribution = Hash.new(0)
      type_distribution = Hash.new(0)

      @deck_cards.each do |card_entry|
        count = card_entry[:count]
        card = card_entry[:card]
        total_cards += count

        # マナカーブ
        if card["manaValue"]
          cmc = card["manaValue"].to_i
          mana_curve[cmc] += count
        end

        # 色分布
        if card["colors"]
          colors = extract_colors(card)
          if colors.empty?
            color_distribution["Colorless"] += count
          else
            colors.each do |color|
              color_distribution[color] += count
            end
          end
        end

        # タイプ分布
        type = card["type"]
        if type.include?("Land")
          type_distribution["Land"] += count
        elsif type.include?("Creature")
          type_distribution["Creature"] += count
        elsif type.include?("Instant")
          type_distribution["Instant"] += count
        elsif type.include?("Sorcery")
          type_distribution["Sorcery"] += count
        elsif type.include?("Enchantment")
          type_distribution["Enchantment"] += count
        elsif type.include?("Artifact")
          type_distribution["Artifact"] += count
        elsif type.include?("Planeswalker")
          type_distribution["Planeswalker"] += count
        else
          type_distribution["Other"] += count
        end
      end

      {
        card_count: total_cards,
        mana_curve: mana_curve.sort.to_h,
        color_distribution: color_distribution,
        type_distribution: type_distribution
      }
    end

    # カードの色情報を安全に抽出
    def extract_colors(card)
      return [] unless card["colors"]

      # colorsが文字列の場合（JSON形式かどうかチェック）
      if card["colors"].is_a?(String)
        begin
          # JSONとしてパースを試みる
          if card["colors"].strip.start_with?("[") && card["colors"].strip.end_with?("]")
            JSON.parse(card["colors"])
          else
            # 単一の色名の場合（例: "G"）
            [card["colors"]]
          end
        rescue JSON::ParserError
          # パースに失敗した場合、文字列をそのまま使用
          if card["colors"].length == 1
            [card["colors"]] # 単一文字は単一の色と解釈
          else
            card["colors"].chars # 複数文字列は各文字を色と解釈
          end
        end
      elsif card["colors"].is_a?(Array)
        # すでに配列の場合はそのまま返す
        card["colors"]
      else
        # その他の型の場合は空配列
        []
      end
    end

    # デッキ内のカードを検索
    def search_in_deck(keyword)
      @deck_cards.select do |card_entry|
        card = card_entry[:card]
        card["jp_name"]&.include?(keyword) ||
          card["en_name"]&.include?(keyword) ||
          card["jp_text"]&.include?(keyword) ||
          card["en_text"]&.include?(keyword) ||
          card["type"]&.include?(keyword)
      end
    end

    # デッキリストをテキスト形式で保存
    def save_deck_to_file(filename)
      File.open(filename, "w") do |f|
        @deck_cards.each do |card_entry|
          f.puts "#{card_entry[:count]}x #{card_entry[:card]["jp_name"] || card_entry[:card]["name"]}"
        end
      end
    end

    # 検索メソッドをQueryから呼び出すためのメソッド
    def find_card_by_jp_name(name)
      @query.find_card_by_jp_name(name)
    end

    def search_cards_by_jp_name(name)
      @query.search_cards_by_jp_name(name)
    end

    def search_cards_by_colors_and_text(colors, text_keywords, max_cmc = nil)
      @query.search_cards_by_colors_and_text(colors, text_keywords, max_cmc)
    end

    def search_cards_by_type(type_keyword, colors = [], max_cmc = nil)
      @query.search_cards_by_type(type_keyword, colors, max_cmc)
    end

    def find_green_draw_cards(max_cmc = 3)
      draw_keywords = [
        "draw a card",
        "draw cards",
        "search library",
        "put into your hand"
      ]
      search_cards_by_colors_and_text(["G"], draw_keywords, max_cmc)
    end
  end
end



module Mtg
  COLOR_MAP = {
    "W" => "白",
    "U" => "青",
    "B" => "黒",
    "R" => "赤",
    "G" => "緑",
    "Colorless" => "無色"
  }

  TYPE_MAP = {
    "Land" => "土地",
    "Creature" => "クリーチャー",
    "Instant" => "インスタント",
    "Sorcery" => "ソーサリー",
    "Enchantment" => "エンチャント",
    "Artifact" => "アーティファクト",
    "Planeswalker" => "プレインズウォーカー",
    "Other" => "その他"
  }

  # Factory method to create appropriate database adapter
  def self.create_adapter(type = :sqlite, path_or_db = Constants::DB_FILE_NAME)
    case type
    when :sqlite
      Database::SqliteAdapter.new(path_or_db)
    when :sqljs
      Database::SqljsAdapter.new(path_or_db)
    else
      raise ArgumentError, "Unknown adapter type: #{type}"
    end
  end

  # Factory method to create service with appropriate adapter
  def self.create_service(adapter_type = :sqlite, path_or_db = Constants::DB_FILE_NAME)
    adapter = create_adapter(adapter_type, path_or_db)
    query = Query.new(adapter)
    Service.new(query)
  end
end

# エントリーポイント
if __FILE__ == $PROGRAM_NAME
  Mtg::Cli.new.run
end

# Web application for MTG deck analysis

module Mtg
  class WebApp
    # デフォルトのデッキリスト
    DEFAULT_DECK = <<~DECK
      スパイクの飼育係
      リシュカーの巧技
      無常の神
      活力の温泉
      太っ腹、グラングリー
      忠実な守護者
      奔放の神殿
      巫師の天啓
      獣相のシャーマン
      尖塔の庭
      雄牛のやっかいもの
      オパールの宮殿
      桜族の長老
      骨溜め
      神との協約
      チェーンソー
      秘紋のアルマサウルス
      野生の律動
      速足のブーツ
      冒涜の行動
      大量破壊
      ピーマの改革派、リシュカー
      熊の陰影
      狩人の眼識
      酸のスライム
      連鎖反応
      不屈の自然
      獲物道
      執念の剣
      魔道士殺しの剣
      骨の鋸
      木霊の手の内
      非常識な図体
      怒り狂う山峡
      風変わりな果樹園
      鎮まらない怒り
      再鍛の黒き剣
      内にいる獣
      魂の威厳
      統率の塔
      ブリキ通りの重鎮、クレンコ
      秘儀の印鑑
      舌長ハイドラ
      硬化した鱗
      蛇の陰影
      マナ喰らいのハイドラ
      煙の精霊の助け
      怨恨
      巨森、オラン＝リーフ
      長所食い
      燃えがらの林間地
      耕作
      遥か見
      太陽の指輪
      忘れられた古霊
      漁る軟泥
      大蛇の統合守り
      記憶の仮面
      グルールの芝地
      ナイレアの試練
      野生林の災い魔
      ファーティリド
      モスファイアの谷
      折れ刃のちし郎
      都和市の案内ボット
      稲妻のすね当て
      極楽鳥
      探検者の望遠鏡
      ガラクの蜂起
      新緑の女魔術師
      頭蓋骨絞め
      守護者計画
      バジリスクの首輪
      楽園のドルイド
      14x 森
      12x 山
    DECK

    def initialize
      @document = JS.global[:document]
      @sql = nil
      @mtg_service = nil
      @current_deck = nil
    end

    def start
      # Wait for SQL.js to be initialized
      init_options = JS.eval("return {locateFile: file => `https://cdnjs.cloudflare.com/ajax/libs/sql.js/1.8.0/${file}`}")
      @sql = JS.global.initSqlJs(init_options).await

      setup_ui
      setup_event_handlers

      update_status("MTGデッキ分析ツールへようこそ！データベースファイルをアップロードしてください。")
    end

    private

    def setup_ui
      html = <<~HTML
        <div class="max-w-2xl mx-auto p-4">
          <h1 class="text-2xl font-bold mb-4 text-gray-700">MTG デッキ分析</h1>
          
          <div id="status" class="bg-gray-100 p-3 rounded mb-4 text-gray-700"></div>
          
          <div class="bg-white border-2 border-dashed border-gray-300 hover:border-gray-400 p-6 rounded-lg mb-4 text-center transition-colors">
            <svg class="mx-auto h-12 w-12 text-gray-400 mb-3" stroke="currentColor" fill="none" viewBox="0 0 48 48" aria-hidden="true">
              <path d="M28 8H12a4 4 0 00-4 4v20m32-12v8m0 0v8a4 4 0 01-4 4H12a4 4 0 01-4-4v-4m32-4l-3.172-3.172a4 4 0 00-5.656 0L28 28M8 32l9.172-9.172a4 4 0 015.656 0L28 28m0 0l4 4m4-24h8m-4-4v8m-12 4h.02" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" />
            </svg>
            <label for="fileInput" class="text-lg font-medium text-gray-700 block mb-1">データベースファイルをアップロード</label>
            <p class="text-sm text-gray-500 mb-3">AllPrintings.sqlite をドラッグ&ドロップまたは選択</p>
            <input type="file" id="fileInput" accept=".sqlite,.db" class="file:mr-4 file:py-2 file:px-4 file:rounded file:border-0 file:text-sm file:font-semibold file:bg-gray-100 file:text-gray-700 hover:file:bg-gray-200 cursor-pointer">
          </div>
          
          <div id="main-content" style="display: none;">
            <div class="border border-gray-300 p-4 rounded bg-white">
              <h2 class="text-xl font-semibold mb-3 text-gray-600">デッキリスト</h2>
              <div class="mb-3">
                <textarea id="deckInput" class="w-full h-96 p-2 border border-gray-300 rounded font-mono text-sm">#{DEFAULT_DECK.strip}</textarea>
              </div>
              <button id="analyzeButton" class="bg-green-600 text-white px-4 py-2 rounded hover:bg-green-700">デッキを分析</button>
              
              <div class="mt-4">
                <h3 class="text-lg font-medium mb-2 text-gray-600">デッキ内検索</h3>
                <div class="flex">
                  <input type="text" id="deckSearch" placeholder="キーワードを入力" class="flex-1 p-1 border border-gray-300 rounded mr-2">
                  <button id="deckSearchButton" class="bg-gray-600 text-white px-3 py-1 rounded hover:bg-gray-700">検索</button>
                </div>
              </div>
              
              <div id="analysisResults" class="mt-4"></div>
            </div>
          </div>
        </div>
      HTML

      @document[:body][:innerHTML] = html
    end

    def setup_event_handlers
      # File upload handler
      file_input = @document.getElementById("fileInput")

      file_input.addEventListener("change", proc { |e|
        Fiber.new do
          target = e[:target]
          files = target[:files]
          file = files[0]
          return unless file

          process_database_file(file)
        rescue => error
          update_status("ファイル読み込みエラー: #{error.message}")
        end.transfer
      })

      # Analyze button handler
      analyze_button = @document.getElementById("analyzeButton")

      analyze_button.addEventListener("click", proc { |e|
        begin
          analyze_deck
        rescue => error
          update_status("分析エラー: #{error.message}")
        end
      })

      # Deck search button handler
      deck_search_button = @document.getElementById("deckSearchButton")

      deck_search_button.addEventListener("click", proc { |e|
        begin
          search_in_deck
        rescue => error
          update_status("検索エラー: #{error.message}")
        end
      })
    end

    def process_database_file(file)
      update_status("データベースを読み込んでいます...")

      begin
        # Read file as array buffer
        buffer = file.arrayBuffer.await
        uint8_array = JS.global[:Uint8Array].new(buffer)

        # Create SQL.js database with the initialized sql object
        db = @sql[:Database].new(uint8_array)

        # Create adapter and service
        adapter = Database::SqljsAdapter.new(db)
        @mtg_service = Service.new(Query.new(adapter))

        # Show main content
        @document.getElementById("main-content")[:style][:display] = "block"

        update_status("データベースを読み込みました。デッキを分析できます。")
      rescue => e
        error_msg = "エラー: #{e.message}"
        update_status(error_msg)
      end
    end

    def analyze_deck
      return unless @mtg_service

      begin
        deck_input_element = @document.getElementById("deckInput")
        deck_text = deck_input_element[:value].to_s

        return if deck_text.strip.empty?

        update_status("デッキを分析中...")

        # Load deck
        @mtg_service.load_deck_from_text!(deck_text)
        @current_deck = @mtg_service.deck_cards

        # Analyze
        analysis = @mtg_service.analyze_deck

        display_analysis_results(analysis)
        update_status("分析完了")
      rescue => e
        error_msg = "分析エラー: #{e.message}"
        update_status(error_msg)
      end
    end

    def display_analysis_results(analysis)

      results_div = @document.getElementById("analysisResults")

      html = "<h3>デッキ分析結果</h3>"
      html += "<p><strong>総カード枚数: #{analysis[:card_count]}</strong></p>"

      # Mana curve
      html += "<h4>マナカーブ</h4>"
      html += "<div style='margin-bottom: 20px; background: #f8f8f8; padding: 10px; border-radius: 5px;'>"

      max_count = analysis[:mana_curve].values.max || 1
      analysis[:mana_curve].sort.each do |cmc, count|
        bar_width = (count.to_f / max_count * 300).to_i
        percentage = (count.to_f / analysis[:card_count] * 100).round(1)
        html += "<div style='margin: 5px 0;'>"
        html += "<span style='display: inline-block; width: 30px;'>#{cmc}:</span>"
        html += "<span style='display: inline-block; background: #4CAF50; height: 20px; width: #{bar_width}px; margin: 0 10px;'></span>"
        html += "<span>#{count}枚 (#{percentage}%)</span>"
        html += "</div>"
      end
      html += "</div>"

      # Color distribution
      html += "<h4>色分布</h4>"
      html += "<div style='margin-bottom: 20px; background: #f8f8f8; padding: 10px; border-radius: 5px;'>"
      analysis[:color_distribution].sort_by { |color, count| -count }.each do |color, count|
        color_name = COLOR_MAP[color] || color
        percentage = (count.to_f / analysis[:card_count] * 100).round(1)
        html += "<div>#{color_name}: #{count}枚 (#{percentage}%)</div>"
      end
      html += "</div>"

      # Type distribution
      html += "<h4>タイプ分布</h4>"
      html += "<div style='margin-bottom: 20px; background: #f8f8f8; padding: 10px; border-radius: 5px;'>"
      analysis[:type_distribution].sort_by { |type, count| -count }.each do |type, count|
        type_name = TYPE_MAP[type] || type
        percentage = (count.to_f / analysis[:card_count] * 100).round(1)
        html += "<div>#{type_name}: #{count}枚 (#{percentage}%)</div>"
      end
      html += "</div>"

      # Card list
      html += "<h4>カードリスト</h4>"
      html += "<div style='max-height: 300px; overflow-y: auto; background: #f8f8f8; padding: 10px; border-radius: 5px;'>"

      # Group by type
      grouped = @current_deck.group_by { |entry| entry[:card]["type"]&.split(" — ")&.first || "Other" }

      grouped.sort.each do |type, cards|
        html += "<div style='margin-bottom: 10px;'>"
        html += "<strong>#{TYPE_MAP[type] || type}:</strong><br>"
        cards.sort_by { |e| [e[:card]["manaValue"] || 0, e[:card]["jp_name"] || e[:card]["name"]] }.each do |entry|
          card = entry[:card]
          count = entry[:count]
          name = card["jp_name"] || card["name"]
          mana_cost = card["manaCost"] || ""
          html += "<div style='margin-left: 20px;'>#{count}x #{name} #{mana_cost}</div>"
        end
        html += "</div>"
      end
      html += "</div>"

      results_div[:innerHTML] = html
    end

    def search_in_deck
      return unless @current_deck

      keyword = @document.getElementById("deckSearch")[:value].to_s
      return if keyword.strip.empty?

      results = @mtg_service.search_in_deck(keyword)

      results_div = @document.getElementById("analysisResults")
      html = "<h3>デッキ内検索結果: 「#{keyword}」</h3>"

      if results.empty?
        html += "<p>該当するカードはありません。</p>"
      else
        html += "<div style='background: #f8f8f8; padding: 10px; border-radius: 5px;'>"
        results.each do |entry|
          card = entry[:card]
          count = entry[:count]
          name = card["jp_name"] || card["name"]
          text = card["jp_text"] || card["text"] || ""

          # Highlight keyword
          highlighted_text = text.gsub(keyword, "<mark>#{keyword}</mark>")

          html += <<~CARD
            <div style="margin-bottom: 15px; padding: 10px; background: white; border-radius: 5px;">
              <strong>#{count}x #{name}</strong><br>
              <div style="font-size: 0.9em; margin-top: 5px;">#{highlighted_text}</div>
            </div>
          CARD
        end
        html += "</div>"
      end

      results_div[:innerHTML] = html
    end

    def update_status(message)
      @document.getElementById("status")[:innerHTML] = message
    end
  end
end

# Initialize web application
app = Mtg::WebApp.new
app.start
